#include "board.hpp"

namespace sudoku {

namespace {

inline size_t offset(const std::slice &slice, size_t i)
{
	return slice.start() + i * slice.stride();
}

inline size_t offset(const std::gslice &slice, size_t i)
{
	return slice.start() +
	    (i % slice.size()[0]) * slice.stride()[0] +
	    (i / slice.size()[0]) * slice.stride()[1];
}

inline size_t slice_size(const std::slice &slice)
{
	return slice.size();
}

inline size_t slice_size(const std::gslice &slice)
{
	return slice.size()[0] * slice.size()[1];
}

template <class Slice>
std::valarray<bool>
possible_values(const Board &board, const Slice &slice)
{
	std::valarray<bool> fields(true, slice_size(slice));
	for (size_t i = 0; i < fields.size(); i++) {
		Board::value_type val = board.board[ offset(slice, i) ];
		if (val != -1)
			fields[val] = false;
	}
	return fields;
}

}

Board::Board(const Board &b) :
    board(b.board),
    possibilities(b.possibilities),
    _major(b._major),
    _minor(b._minor)
{}

void
Board::swap(Board &b)
{
	std::swap(board, b.board);
	std::swap(possibilities, b.possibilities);
	std::swap(_major, b._major);
	std::swap(_minor, b._minor);
}

void
Board::init_poss()
{
	for (size_t i = 0; i < board.size(); i++) {
		if (board[i] >= 0) continue;

		// ! posh spice
		std::slice poss_slice = poss(i);
		possibilities[poss_slice] = possible_values(*this,
		    row(which_row(i)));
		possibilities[poss_slice] &= possible_values(*this,
		    col(which_col(i)));
		possibilities[poss_slice] &= possible_values(*this,
		    square(which_square(i)));
	}
}

}
