#include <cctype>
#include <iostream>
#include <iterator>
#include <sstream>
#include <string>

#include "board.hpp"
#include "solve.hpp"
#include "translate.hpp"
#include "transpose.hpp"

void print(const std::vector<char> &vec)
{
	size_t rows = std::sqrt(vec.size());
	for (size_t r = 0; r < rows; r++) {
		for (size_t c = 0; c < rows; c++) {
			if (c)
				std::cout << ' ';
			std::cout << vec[r * rows + c];
		}
		std::cout << '\n';
	}
}

int main(int argc, char **argv)
{
	using namespace sudoku;

	if (argc != 2)  {
		std::cout << "usage: " << *argv << " NUMBERS\n"
		    << "where NUMBERS is a string of numbers and '.'\n";
		return 1;
	}

	std::istringstream in(argv[1]);
	std::vector<char> text_in;

	// append all non-whitespace characters to 'text_in'
	for (;;) {
		char c;
		in >> c;
		if (!in) break;
		if (isspace(c)) continue;
		text_in.push_back(c);
	}

	std::string alphabet;
	switch (text_in.size()) {
	case 81:
		alphabet = "123456789";
		break;
	case 256:
		alphabet = "0123456789abcdef";
		break;
	default:
		std::cerr << *argv
		    << ": either too many or too few numbers ("
		    << text_in.size() << ")\n";
		return 1;
	}

	std::vector<int> numbers_in(text_in.size());
	decode(text_in.begin(), text_in.end(),
	    numbers_in.begin(), alphabet);
	transpose_square(numbers_in.begin(), numbers_in.end());
	Board board(numbers_in.begin(), numbers_in.end());

	std::list<boost::shared_ptr<Board> > solutions;
	solve(&solutions, board);

	if (!solutions.size()) {
		std::cout << "no solution\n";
		return 0;
	}

	bool first = true;
	for (std::list<boost::shared_ptr<Board> >::iterator i =
	    solutions.begin(); i != solutions.end(); ++i) {
		if (first)
			first = false;
		else
			std::cout << '\n';
		std::vector<char> text_out((*i)->board.size());
		encode(
		    &((*i)->board[0]),
		    &((*i)->board[0]) + (*i)->board.size(),
		    text_out.begin(), alphabet);
		transpose_square(text_out.begin(), text_out.end());
		print(text_out);
	}

	return 0;
}
