#ifndef SUDOKU_TRANSLATE_HPP
#define SUDOKU_TRANSLATE_HPP

#include <algorithm>
#include <exception>
#include <functional>
#include <sstream>
#include <string>
#include <vector>

#include <iostream>

namespace sudoku {

struct Not_in_alphabet : std::exception {
	Not_in_alphabet(char c) :
	    c(c), n(-1)
	{
		std::ostringstream out;
		out << "character '" << c << "' not in alphabet";
		msg = out.str();
	}

	Not_in_alphabet(int n) :
	    c(0), n(n)
	{
		std::ostringstream out;
		out << "index '" << n << "' not in alphabet";
		msg = out.str();
	}

	~Not_in_alphabet() throw() {}

	const char *what() const throw()
	{ return msg.c_str(); }

	std::string msg;
	char c;
	int n;
};

struct Translate_from : std::unary_function<char, int> {
	Translate_from(const std::string &alphabet) : alphabet(alphabet) {}
	int operator()(char c) const
	{
		if (c == '.') return -1;
		std::string::const_iterator i =
		    std::find(alphabet.begin(), alphabet.end(), c);
		if (i == alphabet.end())
			throw Not_in_alphabet(c);
		return i - alphabet.begin();
	}
	const std::string &alphabet;
};

struct Translate_to : std::unary_function<int, char> {
	Translate_to(const std::string &alphabet) : alphabet(alphabet) {}
	char operator()(int n) const
	{
		if (n == -1) return '.';
		if (n > static_cast<int>(alphabet.size()) - 1)
			throw Not_in_alphabet(n);
		return alphabet[n];
	}
	const std::string &alphabet;
};

template <typename In, typename Out>
Out decode(In start, In end, Out res, const std::string &alphabet)
    throw(Not_in_alphabet)
{
	return std::transform(start, end, res, Translate_from(alphabet));
}

template <typename In, typename Out>
Out encode(In start, In end, Out res, const std::string &alphabet)
    throw(Not_in_alphabet)
{
	return std::transform(start, end, res, Translate_to(alphabet));
}

// assumes v.size() is square
void transpose(std::vector<int> &v);

}

#endif
